@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: clientutil.c,v 1.12 2002/01/20 13:16:55 priikone Exp $ */
+/* $Id: clientutil.c,v 1.13 2002/01/29 20:24:41 priikone Exp $ */
 
 #include "module.h"
 
@@ -303,14 +303,16 @@ int silc_client_check_silc_dir()
   memset(file_public_key, 0, sizeof(file_public_key));
   memset(file_private_key, 0, sizeof(file_private_key));
 
+  identifier = silc_client_create_identifier();
+
   pw = getpwuid(getuid());
   if (!pw) {
     fprintf(stderr, "silc: %s\n", strerror(errno));
+    if (identifier)
+      silc_free(identifier);
     return FALSE;
   }
 
-  identifier = silc_client_create_identifier();
-
   /* We'll take home path from /etc/passwd file to be sure. */
   snprintf(filename, sizeof(filename) - 1, "%s/.silc/", pw->pw_dir);
   snprintf(servfilename, sizeof(servfilename) - 1, "%s/.silc/serverkeys", 
