@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: client_prvmsg.c,v 1.26 2001/12/09 17:58:37 priikone Exp $ */
+/* $Id: client_prvmsg.c,v 1.28 2002/01/31 21:31:44 priikone Exp $ */
 /* This file includes the private message sending and receiving routines
    and private message key handling routines. */
 
@@ -147,7 +147,6 @@ void silc_client_private_message(SilcClient client,
 {
   SilcClientConnection conn = (SilcClientConnection)sock->user_data;
   SilcPrivateMessagePayload payload = NULL;
-  SilcIDCacheEntry id_cache = NULL;
   SilcClientID *remote_id = NULL;
   SilcClientEntry remote_client;
   SilcMessageFlags flags;
@@ -161,14 +160,9 @@ void silc_client_private_message(SilcClient client,
     goto out;
 
   /* Check whether we know this client already */
-  if (!silc_idcache_find_by_id_one_ext(conn->client_cache, (void *)remote_id, 
-				       NULL, NULL, 
-				       silc_hash_client_id_compare, NULL,
-				       &id_cache) || 
-      ((SilcClientEntry)id_cache->context)->nickname == NULL) {
-
-    if (id_cache && id_cache->context) {
-      remote_client = (SilcClientEntry)id_cache->context;
+  remote_client = silc_client_get_client_by_id(client, conn, remote_id);
+  if (!remote_client || !remote_client->nickname) {
+    if (remote_client) {
       if (remote_client->status & SILC_CLIENT_STATUS_RESOLVING) {
 	remote_client->status &= ~SILC_CLIENT_STATUS_RESOLVING;
 	goto out;
@@ -183,8 +177,6 @@ void silc_client_private_message(SilcClient client,
     return;
   }
 
-  remote_client = (SilcClientEntry)id_cache->context;
-
   /* Parse the payload and decrypt it also if private message key is set */
   payload = silc_private_message_payload_parse(packet->buffer->data,
 					       packet->buffer->len,
