@@ -19,7 +19,7 @@
 /*
  * Created: Fri Jul 25 18:52:14 1997
  */
-/* $Id: silcpacket.c,v 1.46 2001/12/20 15:24:15 priikone Exp $ */
+/* $Id: silcpacket.c,v 1.47 2002/01/26 19:38:15 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -56,7 +56,8 @@ int silc_packet_send(SilcSocketConnection sock, bool force_send)
     ret = silc_socket_write(sock);
 
     if (ret == -1) {
-      SILC_LOG_ERROR(("Error sending packet, dropped"));
+      SILC_LOG_ERROR(("Error sending packet, dropped: %s", 
+                      strerror(errno)));
     }
     if (ret != -2)
       return ret;
